###
# Copyright (c) 2011, Ricky Zhou
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions, and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions, and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the author of this software nor the name of
#     contributors to this software may be used to endorse or promote products
#     derived from this software without specific prior written consent.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

###

import supybot.utils as utils
from supybot.commands import *
import supybot.plugins as plugins
import supybot.ircutils as ircutils
import supybot.callbacks as callbacks

import supybot.conf as conf
import supybot.registry as registry
import supybot.ircmsgs as ircmsgs

import re

class PingLists(callbacks.Plugin):
    '''
    Ping lists plugin.
    '''

    def put_list(self, name, nick_set):
        # TODO this is all full of race conditions.
        pinglist_group = self.registryValue('pinglists', value=False)
        if name in self.pinglists:
            pinglist_group.unregister(name)

        conf.registerGlobalValue(pinglist_group, name,
            registry.SpaceSeparatedSetOfStrings(nick_set, ''))

        self.pinglists[name] = nick_set

    def __init__(self, irc):
        super(PingLists, self).__init__(irc)

        self.pinglists = {}

        group = conf.supybot.plugins.PingLists.pinglists

        for (name, alias) in registry._cache.iteritems():
            name = name.lower()
            if name.startswith('supybot.plugins.pinglists.pinglists.'):
                name = name[len('supybot.plugins.pinglists.pinglists.'):]
                if '.' in name:
                    continue
                conf.registerGlobalValue(group, name,
                    registry.SpaceSeparatedSetOfStrings(set(), ''))

        for (name, value) in group.getValues(fullNames=False):
            self.pinglists[name] = value()

            for (name, value) in self.pinglists.iteritems():
                try:
                    self.put_list(name, value)
                except Exception, e:
                    self.log.exception('Exception when trying to add pinglist %s.  '
                                       'Reomving from the PingLists database.', name)
                    del self.pinglists[name]

    _listname_regex = re.compile(r'\A\w+\Z')
    def _validate_listname(self, name):
        if not self._listname_regex.match(name):
            return False
        return True

    def add(self, irc, msg, args, listname, nicks):
        '''<listname> [<nick> ...]

        Adds a list of nicks to <listname>.  If no nicks are given, adds
        the caller's nick to the list.
        '''
        try:
            pinglist = self.pinglists[listname]
        except KeyError:
            irc.error('No such ping list.')
            return

        if nicks is None:
            nicks = set([msg.nick])
        else:
            nicks = set(nicks)
            for nick in nicks:
                if not ircutils.isNick(nick):
                    irc.error('Invalid nick: %s' % nick)
                    return

        nick_set = self.pinglists[listname] | nicks
        self.put_list(listname, nick_set)

        irc.reply('Nick(s) successfully added.')

    add = wrap(add, ['something', optional(many('something'))])

    def remove(self, irc, msg, args, listname, nicks):
        '''<listname> [<nick> ...]

        Removes a list of nicks from <listname>.  If no nicks are given,
        removes the caller's nick from the list.
        '''
        try:
            pinglist = self.pinglists[listname]
        except KeyError:
            irc.error('No such ping list.')
            return

        if nicks is None:
            nicks = set([msg.nick])
        else:
            nicks = set(nicks)
            for nick in nicks:
                if not ircutils.isNick(nick):
                    irc.error('Invalid nick: %s' % nick)
                    return

        pinglist_group = self.registryValue('pinglists', value=False)

        nick_set = self.pinglists[listname] - nicks
        self.put_list(listname, nick_set)

        irc.reply('Nick(s) successfully removed.')

    remove = wrap(remove, ['something', optional(many('something'))])

    def create(self, irc, msg, args, listname, nicks):
        '''<listname> [<nick> ...]

        Creates a new pinglist called <listname>.  An optional list of
        initial nicks may be given.
        '''
        if not self._validate_listname(listname):
            irc.error('List names can only contain alphanumeric characters.')
            return

        if listname in self.pinglists:
            irc.error('A list with that name already exists.')
            return

        if nicks is None:
            nick_set = set()
        else:
            nick_set = set(nicks)
            for nick in nick_set:
                if not ircutils.isNick(nick):
                    irc.error('Invalid nick: %s' % nick)
                    return

        self.put_list(listname, nick_set)

        irc.reply('Pinglist %s added.' % listname)

    create = wrap(create, ['something', optional(many('something'))])

    def delete(self, irc, msg, args, listname):
        '''<listname>

        Deletes <listname>.
        '''
        if not listname in self.pinglists:
            irc.error('No such ping list.')
            return

        pinglist_group = self.registryValue('pinglists', value=False)
        pinglist_group.unregister(listname)
        del self.pinglists[listname]

        irc.reply('Pinglist %s deleted.' % listname)

    delete = wrap(delete, ['something'])

    def show(self, irc, msg, args, listname):
        '''<listname>

        Displays the members of <listname>.
        '''
        try:
            pinglist = self.pinglists[listname]
        except KeyError:
            irc.error('No such ping list.')
            return

        message = 'Members of %s: ' % listname
        message += ' '.join(sorted(pinglist))

        irc.reply(message)

    show = wrap(show, ['something'])

    def _nick_match(self, nick1, nick2):
        if nick1 == nick2:
            return True

        if re.match(r'%s[-_|]' % re.escape(nick1), nick2):
            return True

        return False

    def doping(self, irc, msg, args, listname, channel, message):
        '''<listname> [<channel>] <message>

        Pings all of the members of the <listname> in <channel> with
        <message>.  If no channel is specified, the ping occurs in the
        channel that this was called in.
        '''
        try:
            pinglist = self.pinglists[listname]
        except KeyError:
            irc.error('No such ping list.')
            return

        if not pinglist:
            irc.error('Ping list is empty.')
            return

        # Handle matching of away/status nicks.
        ping_set = set(pinglist)
        for nick in pinglist:
            for nick2 in irc.state.channels[channel].users:
                if nick == nick2:
                    continue
                if self._nick_match(nick, nick2):
                    ping_set.add(nick2)
        pinglist = sorted(ping_set)

        ping_message = ircutils.wrap(' '.join(pinglist), 256)

        irc.queueMsg(ircmsgs.notice(channel, message))
        for line in ping_message:
            irc.queueMsg(ircmsgs.notice(channel, line))
        irc.queueMsg(ircmsgs.notice(channel, message))

    doping = wrap(doping, ['something', 'inChannel', 'text'])

    def list(self, irc, msg, args):
        '''
        Displays a list of all ping lists.
        '''

        message = 'Current ping lists: '
        message += ' '.join(sorted(self.pinglists.keys()))

        irc.reply(message)

    list = wrap(list)

Class = PingLists

# vim:set shiftwidth=4 softtabstop=4 expandtab textwidth=79:
